use futures::{
    channel::mpsc::UnboundedReceiver,
    stream::{Next, StreamExt},
};
use log::error;
use netlink_packet_core::NetlinkMessage;
use netlink_packet_route::RouteNetlinkMessage;
use netlink_sys::{AsyncSocket, SocketAddr};
use rtnetlink::new_connection;
use tokio::task::JoinHandle;

const RTNLGRP_LINK: u32 = 1;
const RTNLGRP_IPV4_IFADDR: u32 = 5;

const fn nl_mgrp(group: u32) -> u32 {
    if group > 31 {
        panic!("use netlink_sys::Socket::add_membership() for this group");
    }
    if group == 0 {
        0
    } else {
        1 << (group - 1)
    }
}

type Messages = UnboundedReceiver<(NetlinkMessage<RouteNetlinkMessage>, SocketAddr)>;
pub struct NetworkChangeListener {
    messages: Messages,
    thread_handle: JoinHandle<()>,
}

impl NetworkChangeListener {
    pub fn new() -> Option<Self> {
        let (mut conn, mut _handle, messages) = new_connection().ok()?;
        let groups = nl_mgrp(RTNLGRP_LINK) | nl_mgrp(RTNLGRP_IPV4_IFADDR);

        let addr = SocketAddr::new(0, groups);

        if let Err(e) = conn.socket_mut().socket_mut().bind(&addr) {
            error!("Failed to bind to socket: {:?}", &e);
            return None;
        }

        let thread_handle = tokio::spawn(conn);
        Some(Self {
            messages,
            thread_handle,
        })
    }

    pub fn next_message(&mut self) -> Next<'_, Messages> {
        self.messages.next()
    }
}

impl Drop for NetworkChangeListener {
    fn drop(&mut self) {
        self.thread_handle.abort();
    }
}
