// SPDX: BSD-2-Clause

use std::time::Duration;

use crate::{get_config_path, read_config, Config};
use log::error;

pub async fn get_config() -> Option<Config> {
    let config_path = match get_config_path().await {
        Ok(path) => path,
        Err(tried_paths) => {
            let extra: String = if !tried_paths.is_empty() {
                let joined = tried_paths
                    .iter()
                    .filter_map(|path| path.to_str())
                    .collect::<Vec<_>>()
                    .join(", ");
                format!(", tried the paths: {}", &joined)
            } else {
                String::with_capacity(0)
            };
            error!("Failed to find any config file{}", &extra);
            return None;
        }
    };

    let read_result = read_config(&config_path).await;
    if let Err(e) = &read_result {
        error!("Error occurred while getting config path: {:?}", e);
    }
    read_result.ok()
}
pub fn duration_to_string(duration: &Duration) -> String {
    let mut secs = duration.as_secs();
    let nanos = duration.subsec_nanos();

    let days = secs / 86400;
    secs %= 86400;
    let hours = secs / 3600;
    secs %= 3600;
    let minutes = secs / 60;
    secs %= 60;

    let mut ret = String::new();
    if days > 0 {
        ret.push_str(&format!("{}d ", days));
    }
    if hours > 0 {
        ret.push_str(&format!("{}h ", hours));
    }
    if minutes > 0 {
        ret.push_str(&format!("{}m ", minutes));
    }
    if secs > 0 {
        ret.push_str(&format!("{}s ", secs));
    }
    if nanos > 0 {
        ret.push_str(&format!("{}ns", nanos));
    }
    ret
}
